__version__ = "0.23.2"

__doc__ = """

Helpful CLI to automate the development workflow.

- Create and manage your local and remote projects

- Build projects from cookiecutter templates.

- Easily create/manage virtual environments.

- Minimal configuration required.
"""
